@@ -53,7 +53,7 @@ analyze_error(tree *node, const char *format, ...)
     return;
 
   va_start(args, format);
-  vsprintf(buffer, format, args);
+  vsnprintf(buffer, sizeof(buffer), format, args);
   va_end(args);
 
   if ((node) && (node->file_id)) {
@@ -80,7 +80,7 @@ analyze_warning(tree *node, const char *format, ...)
     return;
 
   va_start(args, format);
-  vsprintf(buffer, format, args);
+  vsnprintf(buffer, sizeof(buffer), format, args);
   va_end(args);
 
   if ((node) && (node->file_id)) {
@@ -470,6 +470,7 @@ analyze_call(tree *call, gp_boolean in_expr, enum size_tag codegen_size)
   tree *call_args;
   tree *assignment;
   char *return_name;
+  struct variable *return_var;
   enum size_tag call_size;
   gp_boolean load_result = false;
 
@@ -562,6 +563,11 @@ analyze_call(tree *call, gp_boolean in_expr, enum size_tag codegen_size)
     tree *symbol;
 
     return_name = mangle_name2(var->name, "return");
+
+    return_var = get_global(return_name);
+    assert(return_var != NULL);
+    return_var->is_used = true;
+ 
     symbol = mk_symbol(return_name, NULL);
     COPY_DEBUG(call, symbol);
     codegen_expr(symbol, codegen_size);
@@ -734,6 +740,7 @@ analyze_expr(tree *expr)
   gp_boolean constant_offset = true;
   int element_size;
   int offset = 0;
+  tree *unop = NULL;
 
   if ((expr->tag != node_binop) || (BINOP_OP(expr) != op_assign)) {
     analyze_error(expr, "expression is missing =");
@@ -788,11 +795,17 @@ analyze_expr(tree *expr)
   /* optimize the expression */
   right = optimize_expr(right);
 
-  /* write the expression */
-  codegen_expr(right, size);
+  unop = optimize_unop_expr(var, left, right);
+  if (unop) {
+    /* write the simplified expression */
+    codegen_unop(var, constant_offset, offset, SYM_OFST(left), unop, size);
+  } else {
+    /* write the expression */
+    codegen_expr(right, size);
 
-  /* put the result in memory */
-  codegen_store(var, constant_offset, offset, SYM_OFST(left));
+    /* put the result in memory */
+    codegen_store(var, constant_offset, offset, SYM_OFST(left));
+  }
 
   /* mark the symbol as having a valid value */
   var->is_assigned = true;
@@ -828,6 +841,7 @@ analyze_assembly(tree *assembly)
 void
 analyze_statements(tree *statement)
 {
+  struct variable *label_var;
 
   if (statement == NULL)
     return;
@@ -843,6 +857,28 @@ analyze_statements(tree *statement)
     case node_cond:
       analyze_cond(statement, NULL);  
       break;
+    case node_goto:
+      label_var = get_global(GOTO_NAME(statement));
+      if (label_var != NULL) {
+        if (label_var->tag == sym_label) {
+          codegen_jump(label_var->name);
+        } else {
+          analyze_error(statement, "%s is not a label", GOTO_NAME(statement));  
+        }
+      } else {
+        analyze_error(statement, "unknown label %s", GOTO_NAME(statement));  
+      }
+      break;
+    case node_label:
+      label_var = get_global(LABEL_NAME(statement));
+      if (label_var != NULL) {
+        /* errors will be generated in scan_labels() */
+        codegen_write_label(label_var->name);
+        if (label_var->storage == storage_public) {
+          codegen_write_asm("global %s", label_var->name);
+        }
+      }
+      break; 
     case node_loop:
       analyze_loop(statement);
       break; 
@@ -853,7 +889,40 @@ analyze_statements(tree *statement)
       analyze_expr(statement);
     }
     statement = statement->next;
-  } 
+  }
+
+  return;
+}
+
+void
+scan_labels(tree *statement, char *base_name)
+{
+  char *label;
+  struct variable *label_var;
+
+  while(statement) {
+    switch(statement->tag) {
+    case node_cond:
+      scan_labels(COND_BODY(statement), base_name);  
+      break;
+    case node_label:
+      label = mangle_name2(base_name, LABEL_NAME(statement));      
+      label_var = add_global_symbol(label, 
+                                    statement,
+                                    sym_label,
+                                    storage_local,
+                                    NULL);
+      add_symbol_alias(LABEL_NAME(statement), statement, label_var);
+      break; 
+    case node_loop:
+      scan_labels(LOOP_BODY(statement), base_name);  
+      break; 
+    default:
+      /* do nothing */
+      break;
+    }
+    statement = statement->next;
+  }
 
   return;
 }
@@ -1021,6 +1090,9 @@ analyze_procedure(tree *procedure, gp_boolean is_func)
         /* initialize the local data */
         append_init(decl_name, DECL_INIT(decl));
       }
+      if (DECL_ADDR(decl)) {
+        analyze_error(decl, "address can't be specified on local data");
+      }
     }
 
     decl = decl->next;
@@ -1037,6 +1109,7 @@ analyze_procedure(tree *procedure, gp_boolean is_func)
     codegen_write_asm("banksel %s", LOCAL_DATA_LABEL);
     codegen_write_asm("bankisel %s", LOCAL_DATA_LABEL);
   }
+  scan_labels(statements, var->name);
   analyze_statements(statements);
   codegen_finish_proc(!generating_function);
 
@@ -1067,12 +1140,17 @@ analyze_constants(void)
   for (i = 0; i < HASH_SIZE; i++) {
     for (sym = state.global->hash_table[i]; sym; sym = sym->next) {
       var = get_symbol_annotation(sym);
-      if (var && (var->tag == sym_const)) {
+      if (var && ((var->tag == sym_const) || 
+                  (var->is_absolute && is_extern(var)))) {
          if (first_time) {
            codegen_write_comment("constants");
            first_time = false;
          }
-         codegen_write_equ(var->name, var->value); 
+         if (var->tag == sym_const) {
+           codegen_write_equ(var->name, var->value);
+         } else if (var->is_absolute) {
+           codegen_write_equ(var->name, var->address);
+         }
       }      
     }
   }
@@ -1098,7 +1176,7 @@ analyze_declarations(void)
   for (i = 0; i < HASH_SIZE; i++) {
     for (sym = state.global->hash_table[i]; sym; sym = sym->next) {
       var = get_symbol_annotation(sym);
-      if (var && is_data(var) && in_module(var)) {
+      if (var && is_data(var) && in_module(var) && !var->is_absolute) {
         write_mem = true;
         if ((!var->is_used) && (var->storage != storage_public)) {
           analyze_warning(var->node, "unused variable %s", var->name);
@@ -1119,6 +1197,32 @@ analyze_declarations(void)
   if (first_time == false)
     fprintf(state.output.f, "\n");
 
+  /* write the abolute sections memory section */
+
+  first_time = true;
+
+  for (i = 0; i < HASH_SIZE; i++) {
+    for (sym = state.global->hash_table[i]; sym; sym = sym->next) {
+      var = get_symbol_annotation(sym);
+      if (var && is_data(var) && in_module(var) && var->is_absolute) {
+        write_mem = true;
+        if ((!var->is_used) && (var->storage != storage_public)) {
+          analyze_warning(var->node, "unused variable %s", var->name);
+          if ((state.optimize.unused_mem) && (!var->is_assigned)) {
+            write_mem = false;
+          }
+        }
+        if (write_mem) {
+          fprintf(state.output.f, ".udata_%s udata %#x\n",
+                  var->name,
+                  var->address);
+          codegen_write_data(var->name, type_size(var->type), var->storage);
+          fprintf(state.output.f, "\n");
+        }
+      }      
+    }
+  }
+
   /* write the external symbols */
 
   first_time = true;
@@ -1129,7 +1233,7 @@ analyze_declarations(void)
       assert(var != NULL);
       if (var && 
           ((var->is_used) || (var->is_assigned)) &&
-          is_extern(var)) {
+          is_extern(var) && !var->is_absolute) {
         if (first_time == true) {
           codegen_write_comment("external subprograms and data");
           first_time = false;
@@ -1325,28 +1429,54 @@ analyze_type(tree *file, tree *type, gp_boolean add_alias)
 
 }
 
+typedef void analyze_func_type(tree *file);
+#define ANALYZE_FUNC (*(analyze_func_type*)func_ptr)
+
 static void
-analyze_module(tree *file)
+analyze_preprocess(tree *cond, long int func_ptr)
+{
+
+  if (COND_TEST(cond)) {
+    if (maybe_evaluate(COND_TEST(cond))) {
+      /* the test condition is true, process the body and return */
+      ANALYZE_FUNC(COND_BODY(cond));
+    } else if (COND_NEXT(cond)) { 
+      /* the test condition is false, process the next clause and return */
+      analyze_preprocess(COND_NEXT(cond), func_ptr);
+    }
+  } else {
+    /* there is no test, else equivalent, process the body */
+    ANALYZE_FUNC(COND_BODY(cond));
+  }
+  
+  return;
+}
+
+static void analyze_public(char *public_name);
+
+static void
+analyze_module(tree *current)
 {
-  tree *current;
   char *name;
   char *alias;
   struct variable *var;
 
-  state.module = file;
-
-  current = FILE_BODY(file);
   while (current) {
     switch (current->tag) {
+    case node_alias:
+      break;
+    case node_cond:
+      analyze_preprocess(current, (long int)analyze_module);
+      break;
     case node_pragma:
-      analyze_pragma(current->value.pragma, FILE_TYPE(file));
+      analyze_pragma(current->value.pragma, FILE_TYPE(state.module));
       break;
     case node_type:
-      analyze_type(file, current, true);
+      analyze_type(state.module, current, true);
       break;
     case node_decl:
       alias = DECL_NAME(current);
-      name = mangle_name2(FILE_NAME(file), alias);
+      name = mangle_name2(FILE_NAME(state.module), alias);
       if (DECL_CONSTANT(current)) {
         if (DECL_INIT(current)) {
           var = add_constant(name,
@@ -1367,11 +1497,15 @@ analyze_module(tree *file)
         if (DECL_INIT(current)) {
           analyze_error(current, "initialized data not yet supported");
         }
+        if (DECL_ADDR(current)) {
+          var->is_absolute = true;
+          var->address = maybe_evaluate(DECL_ADDR(current));
+        }
       }
       break;
     case node_proc:
       alias = find_node_name(current);
-      name = mangle_name2(FILE_NAME(file), alias);
+      name = mangle_name2(FILE_NAME(state.module), alias);
       var = add_global_symbol(name, current, sym_proc, storage_private, NULL);
       add_symbol_alias(alias, current, var);
       if (PROC_BODY(current) == NULL) {
@@ -1381,7 +1515,7 @@ analyze_module(tree *file)
       break;
     case node_func:
       alias = find_node_name(current);
-      name = mangle_name2(FILE_NAME(file), alias);
+      name = mangle_name2(FILE_NAME(state.module), alias);
       var = add_global_symbol(name, current, sym_func, storage_private, NULL);
       add_symbol_alias(alias, current, var);
       if (FUNC_BODY(current) == NULL) {
@@ -1389,6 +1523,9 @@ analyze_module(tree *file)
                       "only function definitions are allowed in a module");
       }
       break;
+    case node_with:
+      analyze_public(WITH_NAME(current));
+      break;
     default:
       assert(0);
     }
@@ -1399,105 +1536,117 @@ analyze_module(tree *file)
 }
 
 static void
-analyze_public(tree *file)
+analyze_public(char *public_name)
 {
-  tree *current;
+  tree *current = NULL;
   char *name;
   char *alias;
   struct variable *var;
+  struct symbol *sym;
+  tree *public = NULL;
+  tree *last_module = NULL;
 
-  state.module = file;
+  sym = get_symbol(state.publics, public_name);
+  if (sym) {
+    public = get_symbol_annotation(sym);
+    assert(public != NULL);
+    last_module = state.module;
+    state.module = public;
+    current = FILE_BODY(public);
+  } else {
+    analyze_error(current, "unknown public %s", public_name);
+    return;
+  }
 
-  current = FILE_BODY(file);
   while (current) {
     switch (current->tag) {
+    case node_alias:
+      var = get_global(ALIAS_NAME(current));
+      add_symbol_alias(ALIAS_ALIAS(current), current, var);
+      break;
+    case node_cond:
+      analyze_preprocess(current, (long int)analyze_public);
+      break;
     case node_pragma:
-      analyze_pragma(current->value.pragma,  FILE_TYPE(file));
+      analyze_pragma(current->value.pragma,  FILE_TYPE(state.module));
       break;
     case node_type:
-      if (FILE_TYPE(file) == source_public) {
-        analyze_type(file, current, true);
+      if (FILE_TYPE(state.module) == source_public) {
+        analyze_type(state.module, current, true);
       } else {
-        analyze_type(file, current, false);
+        analyze_type(state.module, current, false);
       }
       break;
-    case node_decl:
-    case node_proc:
-    case node_func:
-      break;
-    default:
-      assert(0);
-    }
-    current = current->next;
-  }
-
-  current = FILE_BODY(file);
-  while (current) {
-    switch (current->tag) {
-    case node_pragma:
-    case node_type:
-      break;
     case node_decl:
       alias = find_node_name(current);
-      name = mangle_name2(FILE_NAME(file), alias);
+      name = mangle_name2(FILE_NAME(state.module), alias);
       if (DECL_CONSTANT(current)) {
         if (DECL_INIT(current)) {
           var = add_constant(name,
                              maybe_evaluate(DECL_INIT(current)),
                              current,
                              DECL_TYPE(current));
-          if (FILE_TYPE(file) == source_with) {
+          if (FILE_TYPE(state.module) == source_with) {
             add_symbol_alias(alias, current, var);
           }
         } else {
           analyze_error(current, "missing constant value");
         }
-      } else if (FILE_TYPE(file) == source_public) {
+      } else if (FILE_TYPE(state.module) == source_public) {
         var = get_global(name);
         if (var) {
           var->storage = storage_public;       
+          if ((DECL_ADDR(current)) && 
+              (var->address != maybe_evaluate(DECL_ADDR(current)))) {
+            analyze_error(current, "mismatch in declared absolute address",
+                          name); 
+          }
         } else {
           analyze_error(current, "missing definition for %s", name); 
         }
         if (DECL_INIT(current)) {
           analyze_error(current, "declarations can not be initialized");
         }
-      } else if (FILE_TYPE(file) == source_with) {
+      } else if (FILE_TYPE(state.module) == source_with) {
         var = add_global_symbol(name,
                                 current,
                                 sym_udata,
-                                FILE_UDATA(file),
+                                FILE_UDATA(state.module),
                                 DECL_TYPE(current));
         if (DECL_INIT(current)) {
           analyze_error(current, "declarations can not be initialized");
         }
+        if (DECL_ADDR(current)) {
+          var->is_absolute = true;
+          var->address = maybe_evaluate(DECL_ADDR(current));
+        }
       } else {
         assert(0);
       }
       break;
     case node_proc:
       alias = find_node_name(current);
-      name = mangle_name2(FILE_NAME(file), alias);
+      name = mangle_name2(FILE_NAME(state.module), alias);
       if (PROC_BODY(current)) {
         analyze_error(current,
                       "only procedure declarations are allowed in a public");
       }
-      if (FILE_TYPE(file) == source_public) {
+      if (FILE_TYPE(state.module) == source_public) {
         var = get_global(name);
         if (var) {
           make_proc_public(var, current);
         } else {
           analyze_error(current, "missing definition for %s", name); 
         }
-      } else if (FILE_TYPE(file) == source_with) {
+      } else if (FILE_TYPE(state.module) == source_with) {
         var = add_global_symbol(name,
                                 current,
                                 sym_proc,
-                                FILE_CODE(file),
+                                FILE_CODE(state.module),
                                 NULL);
         add_arg_symbols(current,
                         var->name,
-                        FILE_UDATA(file),
+                        FILE_UDATA(state.module),
                         false);
       } else {
         assert(0);
@@ -1505,32 +1654,40 @@ analyze_public(tree *file)
       break;
     case node_func:
       alias = find_node_name(current);
-      name = mangle_name2(FILE_NAME(file), alias);
+      name = mangle_name2(FILE_NAME(state.module), alias);
       if (FUNC_BODY(current)) {
         analyze_error(current,
                       "only function declarations are allowed in a public");
       }
-      if (FILE_TYPE(file) == source_public) {
+      if (FILE_TYPE(state.module) == source_public) {
         var = get_global(name);
         if (var) {
           make_proc_public(var, current);
         } else {
           analyze_error(current, "missing definition for %s", name); 
         }
-      } else if (FILE_TYPE(file) == source_with) {
+      } else if (FILE_TYPE(state.module) == source_with) {
         var = add_global_symbol(name,
                                 current,
                                 sym_func,
-                                FILE_CODE(file),
+                                FILE_CODE(state.module),
                                 NULL);
+        add_global_symbol(mangle_name2(var->name, "return"),
+                          current,
+                          sym_udata,
+                          FILE_UDATA(state.module),
+                          FUNC_RET(current));
         add_arg_symbols(current,
                         var->name,
-                        FILE_UDATA(file),
+                        FILE_UDATA(state.module),
                         false);
       } else {
         assert(0);
       }
 
+      break;
+    case node_with:
+      analyze_public(WITH_NAME(current));
       break;
     default:
       assert(0);
@@ -1538,16 +1695,25 @@ analyze_public(tree *file)
     current = current->next;
   }
 
+  state.module = last_module;
+
   return;
 }
 
 static void
-analyze_module_contents(tree *file)
+analyze_module_contents(tree *current)
 {
-  tree *current = FILE_BODY(file);
+  struct variable *var;
 
   while (current) {
     switch (current->tag) {
+    case node_alias:
+      var = get_global(ALIAS_NAME(current));
+      add_symbol_alias(ALIAS_ALIAS(current), current, var);
+      break;
+    case node_cond:
+      analyze_preprocess(current, (long int)analyze_module_contents);
+      break;
     case node_pragma:
     case node_decl:
     case node_type:
@@ -1559,6 +1725,8 @@ analyze_module_contents(tree *file)
     case node_func:
       analyze_procedure(current, 1);
       break;
+    case node_with:
+      break;
     default:
       assert(0);
     }
@@ -1568,40 +1736,48 @@ analyze_module_contents(tree *file)
   return;
 }
 
+/* reset all of the publics to far, allow pragmas to make some near */
+
 void
-analyze(void)
+analyze_setfar(void)
 {
-  tree *current;
-  tree *module = NULL;
-  gp_boolean found_module = false;
+  tree *node;
+
+  node = state.root;
+  while (node) {
+    if (FILE_TYPE(node) == source_with) {
+      FILE_CODE(node) = storage_far;
+      FILE_UDATA(node) = storage_far;
+    }  
+    node = node->next;
+  }
+}
+
+void
+analyze(tree *module)
+{
+  struct symbol *sym;
+  tree *public = NULL;
 
   generating_function = false;
   found_return = false;
   return_size = size_unknown;
 
-  /* add all procedures and data to the global symbol table */
-  current = state.root;
-  while (current) {
-    if (FILE_TYPE(current) == source_module) {
-      if (found_module) {
-        analyze_error(current, "found multiple modules in one file");
-      } else {
-        analyze_module(current);
-        module = current;
-      }
-      found_module = true;
-    }
-    current = current->next;
+  analyze_setfar();
+
+  /* locate the public for the module being compiled */ 
+  sym = get_symbol(state.publics, state.basefilename);
+  if (sym) {
+    public = get_symbol_annotation(sym);
+    FILE_TYPE(public) = source_public;
   }
 
-  /* add all prototypes and externs to global symbol table */
-  current = state.root;
-  while (current) {
-    if ((FILE_TYPE(current) == source_public) ||
-        (FILE_TYPE(current) == source_with)) {
-      analyze_public(current);
-    }
-    current = current->next;
+  /* add all procedures and data to the global symbol table */
+  state.module = module;
+  analyze_module(FILE_BODY(module));
+
+  if (public) {
+    analyze_public(state.basefilename);
   }
 
   if (!state.processor_chosen) {
@@ -1613,13 +1789,11 @@ analyze(void)
   if (gp_num_errors)
     return;
 
-  state.module = module;
-
   /* open the output file */
   codegen_init_asm();
 
   /* scan the module */
-  analyze_module_contents(module);
+  analyze_module_contents(FILE_BODY(module));
 
   /* finish the assembly output */
   codegen_init_data();
@@ -1627,5 +1801,10 @@ analyze(void)
   analyze_constants();
   codegen_close_asm();
 
+  if (public) {
+    /* return the public to with */
+    FILE_TYPE(public) = source_with;
+  }
+
   return;
 }
