@@ -27,6 +27,7 @@ Boston, MA 02111-1307, USA.  */
 #include "analyze.h" 
 #include "codegen.h"
 #include "codegen14.h"
+#include "codegen16e.h"
 
 /* prototypes */
 static void gen_expr(tree *expr);
@@ -47,6 +48,14 @@ typedef void code_func(enum node_op op,
                        int value,
                        char *name);
 
+/* data movement function pointers */
+typedef void code_unop(enum node_op op,
+                       gp_boolean direct,
+                       char *name,
+                       enum size_tag size,
+                       int offset,
+                       gp_boolean add_banksel);
+
 /* load constant function pointer */
 typedef void data_func1(int value, enum size_tag size);
 
@@ -57,6 +66,7 @@ typedef void data_func2(char *name,
                         gp_boolean add_banksel);
 
 #define CODEGEN (*(code_func*)func_ptr->codegen)
+#define UNOPGEN (*(code_unop*)func_ptr->unopgen)
 #define LOAD_CONSTANT (*(data_func1*)func_ptr->load_constant)
 #define LOAD_FILE (*(data_func2*)func_ptr->load_file)
 #define STORE_FILE (*(data_func2*)func_ptr->store_file)
@@ -74,7 +84,7 @@ codegen_write_asm(const char *format, ...)
   char buffer[BUFSIZ]; 
 
   va_start(args, format);
-  vsprintf(buffer, format, args);
+  vsnprintf(buffer, sizeof(buffer), format, args);
   va_end(args);
 
   fprintf(state.output.f, "  %s\n", buffer);
@@ -89,7 +99,7 @@ codegen_write_comment(const char *format, ...)
   char buffer[BUFSIZ]; 
 
   va_start(args, format);
-  vsprintf(buffer, format, args);
+  vsnprintf(buffer, sizeof(buffer), format, args);
   va_end(args);
    
   fprintf(state.output.f, "; %s\n", buffer);
@@ -107,7 +117,7 @@ char *
 codegen_next_label(void)
 {
   char label[BUFSIZ];
-  sprintf(label, "_%i", label_number++);
+  snprintf(label, sizeof(label), "_%i", label_number++);
   return strdup(label);
 }
 
@@ -166,26 +176,44 @@ codegen_indirect(tree *offset,
                  gp_boolean new_expr)
 {
 
-  /* FIXME: the default pointer size needs to be selected (size_uint8 for
-     now ) */
-
   /* indirect access */
   if (new_expr) {
-    codegen_expr(offset, size_uint8);
+    codegen_expr(offset, state.pointer_size);
   } else {
     gen_expr(offset);
   }
   if (var->type->start != 0) {
     /* shift the offset for the first element */
-    CODEGEN(op_add, size_uint8, true, 0 - var->type->start, NULL);
+    CODEGEN(op_add, state.pointer_size, true, 0 - var->type->start, NULL);
   }
   if (element_size != 1) {
     /* scale the offset by the element size */
-    CODEGEN(op_mult, size_uint8, true, element_size, NULL);
+    CODEGEN(op_mult, state.pointer_size, true, element_size, NULL);
   }
-  codegen_write_asm("addlw %s", var->name); 
-  codegen_write_asm("movwf FSR"); 
 
+  switch (state.class) {
+  case PROC_CLASS_PIC14:
+    codegen_write_asm("addlw %s", var->name); 
+    codegen_write_asm("banksel FSR");
+    codegen_write_asm("movwf FSR"); 
+    codegen_write_asm("banksel %s", LOCAL_DATA_LABEL);
+    break;
+  case PROC_CLASS_PIC16E:
+    codegen_write_asm("movf %s, w", WORKING_LABEL);
+    codegen_write_asm("addlw LOW(%s)", var->name); 
+    codegen_write_asm("movff WREG, FSR0L");
+    codegen_write_asm("movf %s + 1, w", WORKING_LABEL);
+    codegen_write_asm("btfsc STATUS, C");
+    codegen_write_asm("addlw 1"); 
+    codegen_write_asm("addlw HIGH(%s)", var->name); 
+    codegen_write_asm("movff WREG, FSR0H");
+    break;
+  default:
+    assert(0);
+  }
+
+
+  return;
 }
 
 /* load the register file into the working register */
@@ -204,14 +232,14 @@ codegen_load_file(tree *symbol, struct variable *var)
       if (can_evaluate(SYM_OFST(symbol), false)) {
         /* direct access */
         offset = analyze_check_array(symbol, var) * element_size;
-        if (var->storage == storage_far) {
+        if (is_far(var)) {
           LOAD_FILE(var->name, codegen_size, offset, true);
         } else {
           LOAD_FILE(var->name, codegen_size, offset, false);
         }
       } else {
         codegen_indirect(SYM_OFST(symbol), var, element_size, false);
-        if (var->storage == storage_far) {
+        if (is_far(var)) {
           LOAD_INDIRECT(var->name, codegen_size, 0, true);
         } else {
           LOAD_INDIRECT(var->name, codegen_size, 0, false);
@@ -222,7 +250,7 @@ codegen_load_file(tree *symbol, struct variable *var)
                     SYM_NAME(symbol));
     }
   } else {
-    if (var->storage == storage_far) {
+    if (is_far(var)) {
       LOAD_FILE(var->name, codegen_size, 0, true);
     } else {
       LOAD_FILE(var->name, codegen_size, 0, false);
@@ -243,20 +271,20 @@ codegen_store(struct variable *var,
 
   if (offset_expr) {
     if (constant_offset) {
-      if (var->storage == storage_far) {
+      if (is_far(var)) {
         STORE_FILE(var->name, codegen_size, offset, true);
       } else {
         STORE_FILE(var->name, codegen_size, offset, false);
       }    
     } else {
-      if (var->storage == storage_far) {
+      if (is_far(var)) {
         STORE_INDIRECT(var->name, codegen_size, 0, true);
       } else {
         STORE_INDIRECT(var->name, codegen_size, 0, false);
       }
     }
   } else {
-    if (var->storage == storage_far) {
+    if (is_far(var)) {
       STORE_FILE(var->name, codegen_size, 0, true);
     } else {
       STORE_FILE(var->name, codegen_size, 0, false);
@@ -433,6 +461,46 @@ codegen_expr(tree *statement, enum size_tag size)
   return;
 }
 
+/* perform the unop on the file register not the working register */
+
+void
+codegen_unop(struct variable *var,
+             gp_boolean constant_offset,
+             int offset,
+             tree *offset_expr,
+             tree *unop,
+             enum size_tag size)
+{
+
+  temp_number = codegen_setup(size);
+
+  if (offset_expr) {
+    if (constant_offset) {
+      if (is_far(var)) {
+        UNOPGEN(UNOP_OP(unop), true, var->name, codegen_size, offset, true);
+      } else {
+        UNOPGEN(UNOP_OP(unop), true, var->name, codegen_size, offset, false);
+      }    
+    } else {
+      if (is_far(var)) {
+        UNOPGEN(UNOP_OP(unop), false, var->name, codegen_size, 0, true);
+      } else {
+        UNOPGEN(UNOP_OP(unop), false, var->name, codegen_size, 0, false);
+      }
+    }
+  } else {
+    if (is_far(var)) {
+      UNOPGEN(UNOP_OP(unop), true, var->name, codegen_size, 0, true);
+    } else {
+      UNOPGEN(UNOP_OP(unop), true, var->name, codegen_size, 0, false);
+    }
+  }
+
+  if (temp_number > max_temp_number)
+    max_temp_number = temp_number;
+
+}
+
 /****************************************************************************/
 /* Write procedures and functions                                           */
 /****************************************************************************/
@@ -523,7 +591,11 @@ codegen_get_temp(enum size_tag size)
 {
   char temp_name[BUFSIZ];
  
-  sprintf(temp_name, "_%s_temp_%d", FILE_NAME(state.module), temp_number);
+  snprintf(temp_name,
+           sizeof(temp_name),
+           "_%s_temp_%d",
+           FILE_NAME(state.module),
+           temp_number);
 
   temp_number += prim_size(size);
 
@@ -540,8 +612,8 @@ codegen_init_asm()
   char buffer[BUFSIZ];
 
   /* open output filename */
-  strcpy(state.asmfilename, state.basefilename);
-  strcat(state.asmfilename, ".asm");
+  strncpy(state.asmfilename, state.basefilename, sizeof(state.asmfilename));
+  strncat(state.asmfilename, ".asm", sizeof(state.asmfilename));
   state.output.f = fopen(state.asmfilename, "w");
   if (state.output.f == NULL) {
     perror(state.asmfilename);
@@ -554,7 +626,7 @@ codegen_init_asm()
   codegen_size = size_uint8;
   codegen_bytes = prim_size(codegen_size);
 
-  gp_date_string(buffer);
+  gp_date_string(buffer, sizeof(buffer));
 
   fprintf(state.output.f, "; %s\n", state.asmfilename);
   fprintf(state.output.f, "; generated by %s on %s\n\n",
@@ -660,11 +732,15 @@ codegen_select(tree *expr)
     break;
   case PROC_CLASS_PIC14:
     func_ptr = &codegen14_func;
+    state.pointer_size = size_uint8;
     break;
   case PROC_CLASS_PIC16:
-  case PROC_CLASS_PIC16E:
     analyze_error(expr, "unsupported processor class");
     break;
+  case PROC_CLASS_PIC16E:
+    func_ptr = &codegen16e_func;
+    state.pointer_size = size_uint16;
+    break;
   default:
     assert(0);
   }
