@@ -22,7 +22,6 @@ Boston, MA 02111-1307, USA.  */
 #ifndef __SCAN_H__
 #define __SCAN_H__
 
-void open_src(char *name, enum source_type type);
-void scan_header(char *file_name);
+void open_src(char *name);
 
 #endif
