@@ -1,5 +1,6 @@
-/* .cod file output
-   Copyright (C) 2001 Craig Franklin
+/* ".COD" file output for gpasm
+   Copyright (C) 2001, 2002, 2003, 2004
+   Craig Franklin
 
 This file is part of gputils.
 
@@ -30,6 +31,5 @@ void cod_init(void);
 void cod_close_file(void);
 void cod_lst_line(int line_type);
 void cod_write_symbols(struct symbol **,int);
-void cod_emit_opcode(int, int);
 
 #endif
