@@ -110,8 +110,8 @@ int endian_swap_word(int x)
  * packed. When two bytes are packed, then the word is emitted.
  */
 
-static int packed_hi_lo = 0;
-static int packed_byte  = 0;
+int packed_hi_lo = 0;
+static int packed_byte = 0;
 
 static void emit_packed(unsigned int value, unsigned int mode)
 {
@@ -323,7 +323,10 @@ static int list_symbol_member(struct pnode *M, struct pnode *L)
     return 0;
   } else if (STRCMP(M->value.symbol, HEAD(L)->value.symbol) == 0) {
     char buf[BUFSIZ];
-    sprintf(buf, "Duplicate macro parameter (%s).", HEAD(L)->value.symbol);
+    snprintf(buf,
+             sizeof(buf), 
+             "Duplicate macro parameter (%s).", 
+             HEAD(L)->value.symbol);
     gperror(GPE_UNKNOWN, buf);
     return 1;
   } else {
@@ -474,6 +477,7 @@ static gpasmVal do_banksel(gpasmVal r,
         address = maybe_evaluate(p);
         bank = gp_processor_check_bank(state.device.class, address);
         state.org += gp_processor_set_bank(state.device.class, 
+                                           state.processor_info->num_banks,
                                            bank, 
                                            state.i_memory, 
                                            state.org);
@@ -486,6 +490,7 @@ static gpasmVal do_banksel(gpasmVal r,
         address = maybe_evaluate(p);
         bank = gp_processor_check_bank(state.device.class, address);
         state.org += gp_processor_set_bank(state.device.class, 
+                                           state.processor_info->num_banks,
                                            bank, 
                                            state.i_memory, 
                                            state.org);
@@ -498,9 +503,17 @@ static gpasmVal do_banksel(gpasmVal r,
         reloc_evaluate(p, RELOCT_BANKSEL);
         emit(0);
       } else {
-        reloc_evaluate(p, RELOCT_BANKSEL);
-        emit(0);
-        emit(0);
+        switch (state.processor_info->num_banks) {
+        case 2:
+          reloc_evaluate(p, RELOCT_BANKSEL);
+          emit(0);
+          break;      
+        case 4:
+          reloc_evaluate(p, RELOCT_BANKSEL);
+          emit(0);
+          emit(0);
+          break;
+        }
       }
     }
   }
@@ -524,14 +537,14 @@ static gpasmVal do_code(gpasmVal r,
     switch (arity) {
     case 0:
       /* new relocatable section */
-      strcpy(state.obj.new_sec_name, ".code");
+      strncpy(state.obj.new_sec_name, ".code", sizeof(state.obj.new_sec_name));
       state.obj.new_sec_addr = 0;
       state.obj.new_sec_flags = STYP_TEXT;
       break;
     case 1:
       /* new absolute section */
       p = HEAD(parms);
-      strcpy(state.obj.new_sec_name, ".code");
+      strncpy(state.obj.new_sec_name, ".code", sizeof(state.obj.new_sec_name));
       state.obj.new_sec_addr = maybe_evaluate(p) >> _16bit_core;
       state.obj.new_sec_flags = STYP_TEXT | STYP_ABS;
       break;
@@ -1107,12 +1120,18 @@ static gpasmVal do_global(gpasmVal r,
       if (p->tag == symbol) {
         s = get_symbol(state.stTop, p->value.symbol);
         if (s == NULL) {
-          sprintf(buf, "Symbol not previously defined (%s).", p->value.symbol);
+          snprintf(buf,
+                   sizeof(buf),
+                   "Symbol not previously defined (%s).",
+                   p->value.symbol);
           gperror(GPE_NOSYM, buf);    
 	} else {
           var = get_symbol_annotation(s);
           if (var == NULL) {
-            sprintf(buf, "Symbol not assigned a value (%s).", p->value.symbol);
+            snprintf(buf,
+                     sizeof(buf),
+                     "Symbol not assigned a value (%s).",
+                     p->value.symbol);
             gpwarning(GPW_UNKNOWN, buf);    
           } else {
             if ((var->previous_type == gvt_address) || 
@@ -1123,8 +1142,10 @@ static gpasmVal do_global(gpasmVal r,
             } else if (var->previous_type == gvt_extern) { 
               gperror(GPE_DUPLAB, NULL);    
             } else {
-              sprintf(buf, "Operand must be an address label (%s).", 
-                      p->value.symbol);
+              snprintf(buf,
+                       sizeof(buf),
+                       "Operand must be an address label (%s).", 
+                       p->value.symbol);
               gperror(GPE_MUST_BE_LABEL, buf);    
             } 
           }         
@@ -1154,14 +1175,14 @@ static gpasmVal do_idata(gpasmVal r,
     switch (arity) {
     case 0:
       /* new relocatable section */
-      strcpy(state.obj.new_sec_name, ".idata");
+      strncpy(state.obj.new_sec_name, ".idata", sizeof(state.obj.new_sec_name));
       state.obj.new_sec_addr = 0;
       state.obj.new_sec_flags = STYP_DATA;
       break;
     case 1:
       /* new absolute section */
       p = HEAD(parms);
-      strcpy(state.obj.new_sec_name, ".idata");
+      strncpy(state.obj.new_sec_name, ".idata", sizeof(state.obj.new_sec_name));
       state.obj.new_sec_addr = maybe_evaluate(p) >> _16bit_core;
       state.obj.new_sec_flags = STYP_DATA | STYP_ABS;
       break;
@@ -1429,7 +1450,7 @@ static gpasmVal do_list(gpasmVal r,
 	} else if (strcasecmp(lhs, "t") == 0) {
 	  ; /* Ignore this for now: always wrap long list lines */
 	} else if (strcasecmp(lhs, "w") == 0) {
-          select_errorlevel(p->value.binop.p1->value.constant);
+          select_errorlevel(maybe_evaluate(p->value.binop.p1));
 	} else if (strcasecmp(lhs, "x") == 0) {
 	  if (enforce_simple(p->value.binop.p1))
             select_expand(p->value.binop.p1->value.symbol);
@@ -1613,7 +1634,7 @@ static gpasmVal do_org(gpasmVal r,
       }	else {
         /* Default section name, this will be overwritten if a label is 
            present. */
-        sprintf(state.obj.new_sec_name, ".org_%d", state.obj.org_num++);
+        snprintf(state.obj.new_sec_name, sizeof(state.obj.new_sec_name), ".org_%d", state.obj.org_num++);
         state.obj.new_sec_addr = r;
         state.obj.new_sec_flags = STYP_TEXT | STYP_ABS;
         state.lst.line.linetype = sec;
@@ -1665,6 +1686,7 @@ static gpasmVal do_pagesel(gpasmVal r,
         page = gp_processor_check_page(state.device.class, address);
 
         state.org += gp_processor_set_page(state.device.class, 
+                                           state.processor_info->num_pages,
                                            page, 
                                            state.i_memory, 
                                            state.org);
@@ -1678,6 +1700,7 @@ static gpasmVal do_pagesel(gpasmVal r,
         page = gp_processor_check_page(state.device.class, address);
 
         state.org += gp_processor_set_page(state.device.class, 
+                                           state.processor_info->num_pages,
                                            page, 
                                            state.i_memory, 
                                            state.org);
@@ -1688,9 +1711,17 @@ static gpasmVal do_pagesel(gpasmVal r,
         emit(0);
         emit(0);
       } else {
-        reloc_evaluate(p, RELOCT_PAGESEL_BITS);
-        emit(0);
-        emit(0);
+        switch (state.processor_info->num_pages) {
+        case 2:
+          reloc_evaluate(p, RELOCT_PAGESEL_BITS);
+          emit(0);
+          break;      
+        case 4:
+          reloc_evaluate(p, RELOCT_PAGESEL_BITS);
+          emit(0);
+          emit(0);
+          break;
+        }
       }
     }
 
@@ -1836,14 +1867,14 @@ static gpasmVal do_udata(gpasmVal r,
     switch (arity) {
     case 0:
       /* new relocatable section */
-      strcpy(state.obj.new_sec_name, ".udata");
+      strncpy(state.obj.new_sec_name, ".udata", sizeof(state.obj.new_sec_name));
       state.obj.new_sec_addr = 0;
       state.obj.new_sec_flags = STYP_BSS;
       break;
     case 1:
       /* new absolute section */
       p = HEAD(parms);
-      strcpy(state.obj.new_sec_name, ".udata");
+      strncpy(state.obj.new_sec_name, ".udata", sizeof(state.obj.new_sec_name));
       state.obj.new_sec_addr = maybe_evaluate(p);
       state.obj.new_sec_flags = STYP_BSS | STYP_ABS;
       break;
@@ -1871,14 +1902,18 @@ static gpasmVal do_udata_acs(gpasmVal r,
     switch (arity) {
     case 0:
       /* new relocatable section */
-      strcpy(state.obj.new_sec_name, ".udata_acs");
+      strncpy(state.obj.new_sec_name,
+              ".udata_acs",
+              sizeof(state.obj.new_sec_name));
       state.obj.new_sec_addr = 0;
       state.obj.new_sec_flags = STYP_BSS | STYP_ACCESS;
       break;
     case 1:
       /* new absolute section */
       p = HEAD(parms);
-      strcpy(state.obj.new_sec_name, ".udata_acs");
+      strncpy(state.obj.new_sec_name,
+              ".udata_acs",
+              sizeof(state.obj.new_sec_name));
       state.obj.new_sec_addr = maybe_evaluate(p);
       state.obj.new_sec_flags = STYP_BSS | STYP_ABS | STYP_ACCESS;
       break;
@@ -1906,14 +1941,18 @@ static gpasmVal do_udata_ovr(gpasmVal r,
     switch (arity) {
     case 0:
       /* new relocatable section */
-      strcpy(state.obj.new_sec_name, ".udata_ovr");
+      strncpy(state.obj.new_sec_name,
+              ".udata_ovr",
+              sizeof(state.obj.new_sec_name));
       state.obj.new_sec_addr = 0;
       state.obj.new_sec_flags = STYP_BSS | STYP_OVERLAY;
       break;
     case 1:
       /* new absolute section */
       p = HEAD(parms);
-      strcpy(state.obj.new_sec_name, ".udata_ovr");
+      strncpy(state.obj.new_sec_name,
+              ".udata_ovr",
+              sizeof(state.obj.new_sec_name));
       state.obj.new_sec_addr = maybe_evaluate(p);
       state.obj.new_sec_flags = STYP_BSS | STYP_ABS | STYP_OVERLAY;
       break;
@@ -1941,14 +1980,18 @@ static gpasmVal do_udata_shr(gpasmVal r,
     switch (arity) {
     case 0:
       /* new relocatable section */
-      strcpy(state.obj.new_sec_name, ".udata_shr");
+      strncpy(state.obj.new_sec_name,
+              ".udata_shr",
+              sizeof(state.obj.new_sec_name));
       state.obj.new_sec_addr = 0;
       state.obj.new_sec_flags = STYP_BSS | STYP_SHARED;
       break;
     case 1:
       /* new absolute section */
       p = HEAD(parms);
-      strcpy(state.obj.new_sec_name, ".udata_shr");
+      strncpy(state.obj.new_sec_name,
+              ".udata_shr",
+              sizeof(state.obj.new_sec_name));
       state.obj.new_sec_addr = maybe_evaluate(p);
       state.obj.new_sec_flags = STYP_BSS | STYP_ABS | STYP_SHARED;
       break;
@@ -2126,10 +2169,12 @@ static void emit_check_relative(int insn, int argument, int mask, int range)
 
   /* If the branch is too far then issue an error */
   if ((argument > range) || (argument < -(range+1))) {
-    sprintf(full_message,"Argument out of range (%d not between %d and %d)\n",
-	    argument,
-	    -(range+1),
-	    range);
+    snprintf(full_message,
+             sizeof(full_message),
+             "Argument out of range (%d not between %d and %d)\n",
+             argument,
+             -(range+1),
+             range);
     gperror(GPE_RANGE, full_message);
   }
 
@@ -2886,7 +2931,7 @@ gpasmVal do_insn(char *name, struct pnode *parms)
           gperror(GPE_UNDEF_PROC, NULL);
         } else {
           char mesg[80];
-          sprintf(mesg, "Unknown opcode \"%.40s\"", name);
+          snprintf(mesg, sizeof(mesg), "Unknown opcode \"%.40s\"", name);
           gperror(GPE_UNKNOWN, mesg);
         }
       }
@@ -3098,7 +3143,7 @@ void cblock_expr(struct pnode *s)
 
 void cblock_expr_incr(struct pnode *s, struct pnode *incr)
 {
-  if ((asm_enabled()) &&(can_evaluate_concatenation(s))) {
+  if ((asm_enabled()) && (can_evaluate_concatenation(s))) {
     set_global(evaluate_concatenation(s), state.cblock, PERMANENT, gvt_cblock);
     state.cblock += maybe_evaluate(incr);
   }
