@@ -264,6 +264,7 @@ void build_tables(void)
 
   if (state.has_stack) {
     remove_linker_symbol("_stack");
+    remove_linker_symbol("_stack_end");
   }
 
   /* All of the archives have been scanned.  If there are still missing
@@ -293,7 +294,7 @@ void gplink_open_coff(char *name)
   FILE *coff;
   char file_name[BUFSIZ];
   
-  strcpy(file_name, name);
+  strncpy(file_name, name, sizeof(file_name));
   
   coff = fopen(file_name, "rb");
   if ((coff == NULL) && (strchr(file_name, PATH_CHAR) == 0)) { 
@@ -301,9 +302,9 @@ void gplink_open_coff(char *name)
     int i;
 
     for(i = 0; i < state.numpaths; i++) {
-      strcpy(file_name, state.paths[i]);
-      strcat(file_name, COPY_CHAR);
-      strcat(file_name, name);
+      strncpy(file_name, state.paths[i], sizeof(file_name));
+      strncat(file_name, COPY_CHAR, sizeof(file_name));
+      strncat(file_name, name, sizeof(file_name));
       coff = fopen(file_name, "rb");
       if (coff != NULL) {
         break;
@@ -364,6 +365,7 @@ void show_usage(void)
   printf("  -m, --map                      Output a map file.\n");
   printf("  -o FILE, --output FILE         Alternate name of output file.\n");
   printf("  -q, --quiet                    Quiet.\n");
+  printf("  -r, --use-shared               Use shared memory if necessary.\n");
   printf("  -s FILE, --script FILE         Linker script.\n");
   printf("  -t SIZE, --stack SIZE          Create a stack section.\n");
   printf("  -v, --version                  Show version.\n");
@@ -386,7 +388,7 @@ void show_usage(void)
   exit(0);
 }
 
-#define GET_OPTIONS "?a:cdf:hI:lmo:qs:t:v"
+#define GET_OPTIONS "?a:cdf:hI:lmo:qrs:t:v"
 
   static struct option longopts[] =
   {
@@ -400,6 +402,7 @@ void show_usage(void)
     { "map",         0, 0, 'm' },
     { "output",      1, 0, 'o' },
     { "quiet",       0, 0, 'q' },
+    { "use-shared",  0, 0, 'r' },
     { "script",      1, 0, 's' },
     { "stack",       1, 0, 't' },
     { "version",     0, 0, 'v' },
@@ -419,7 +422,7 @@ int main(int argc, char *argv[])
   gp_init();
 
   /* initialize */
-  gp_date_string(state.startdate);
+  gp_date_string(state.startdate, sizeof(state.startdate));
   state.hex_format = inhx32;
   state.numpaths = 0;
   state.byte_addr = 0;
@@ -495,7 +498,7 @@ int main(int argc, char *argv[])
       state.mapfile = normal;
       break;
     case 'o':
-      strcpy(state.basefilename, optarg);
+      strncpy(state.basefilename, optarg, sizeof(state.basefilename));
       pc = strrchr(state.basefilename, '.');
       if (pc)
         *pc = 0;
@@ -503,6 +506,9 @@ int main(int argc, char *argv[])
     case 'q':
       gp_quiet = 1;
       break;
+    case 'r':
+      gp_relocate_to_shared = true;
+      break;
     case 's':
       state.srcfilename = optarg;
       break;
@@ -535,7 +541,7 @@ int main(int argc, char *argv[])
 
   if(state.basefilename[0] == '\0') {
     /* set default output filename to be a.o, a.hex, a.cod, a.map */
-    strcpy(state.basefilename, "a");
+    strncpy(state.basefilename, "a", sizeof(state.basefilename));
   }
 
   /* Add the library path to the include paths list last, so that the user
@@ -545,12 +551,12 @@ int main(int argc, char *argv[])
   }
 
   /* setup output filenames */
-  strcpy(state.hexfilename, state.basefilename);
-  strcat(state.hexfilename, ".hex");  
-  strcpy(state.mapfilename, state.basefilename);
-  strcat(state.mapfilename, ".map");  
-  strcpy(state.objfilename, state.basefilename);
-  strcat(state.objfilename, ".cof");  
+  strncpy(state.hexfilename, state.basefilename, sizeof(state.hexfilename));
+  strncat(state.hexfilename, ".hex", sizeof(state.hexfilename));
+  strncpy(state.mapfilename, state.basefilename, sizeof(state.mapfilename));
+  strncat(state.mapfilename, ".map", sizeof(state.mapfilename));
+  strncpy(state.objfilename, state.basefilename, sizeof(state.objfilename));
+  strncat(state.objfilename, ".cof", sizeof(state.objfilename));
 
   /* Open all objects and archives in the file list. */ 
   for ( ; optind < argc; optind++) {
@@ -574,9 +580,9 @@ int main(int argc, char *argv[])
       gp_error("linker script not specified and can't determine default script");
       return EXIT_FAILURE; 
     }
-    strcpy(file_name, gp_lkr_path);    
-    strcat(file_name, COPY_CHAR);
-    strcat(file_name, script_name);
+    strncpy(file_name, gp_lkr_path, sizeof(file_name));
+    strncat(file_name, COPY_CHAR, sizeof(file_name));
+    strncat(file_name, script_name, sizeof(file_name));
     gp_message("using default linker script \"%s\"", file_name);
     open_src(file_name, 0);
     yyparse();
@@ -619,6 +625,7 @@ int main(int argc, char *argv[])
 
   if (state.has_stack) {
     add_linker_symbol("_stack");
+    add_linker_symbol("_stack_end");
   }
 
   /* patch raw data with the relocated symbol values */
