@@ -22,6 +22,8 @@ Boston, MA 02111-1307, USA.  */
 #include "stdhdr.h"
 #include "libgputils.h"
 
+gp_boolean gp_relocate_to_shared = false;
+
 /* Two symbol tables are constructed. The first contains the definitions of all
    external symbols in all the object files.  This symbol table is used for
    relocation and linking.  The second table contains all external symbols 
@@ -290,7 +292,7 @@ gp_cofflink_make_stack(gp_object_type *object, int num_bytes)
     i_memory_put(new->data, i, MEM_USED_MASK);
   }
 
-  /* create the symbol for the start address of the table */
+  /* create the symbol for the start address of the stack */
   symbol = gp_coffgen_findsymbol(object, "_stack");
   if ((symbol != NULL) && (symbol->section_number > 0)) {
     gp_error("_stack symbol already exists");
@@ -304,6 +306,20 @@ gp_cofflink_make_stack(gp_object_type *object, int num_bytes)
     symbol->class          = C_EXT;
   }
 
+  /* create the symbol for the end of the stack */
+  symbol = gp_coffgen_findsymbol(object, "_stack_end");
+  if ((symbol != NULL) && (symbol->section_number > 0)) {
+    gp_error("_stack_end symbol already exists");
+  } else {
+    symbol = gp_coffgen_addsymbol(object);
+    symbol->name           = strdup("_stack_end");
+    symbol->value          = num_bytes - 1;
+    symbol->section_number = 1;
+    symbol->section        = new;
+    symbol->type           = T_NULL;
+    symbol->class          = C_EXT;
+  }
+
   return;
 }
 
@@ -438,6 +454,43 @@ gp_cofflink_merge_sections(gp_object_type *object, int byte_addr)
   return;
 }
 
+/* copy data from idata section to the ROM section */
+
+static void
+_copy_rom_section(gp_object_type *object,
+                  gp_section_type *idata,
+                  gp_section_type *rom)
+{
+  int insn;
+  int data;
+  int from;
+  int to;
+  int last;
+
+  if (object->class == PROC_CLASS_PIC16E) {
+    to = rom->address >> 1;
+  } else {
+    to = rom->address;
+  }
+  last = idata->address + idata->size; 
+  if (object->class == PROC_CLASS_PIC16E) {
+    insn = MEM_USED_MASK;
+    for (from = idata->address; from < last; from += 2) {
+      data =  (i_memory_get(idata->data, from) & 0xff);
+      data |= ((i_memory_get(idata->data, from + 1) & 0xff) << 8);
+      i_memory_put(rom->data, to++, insn | data);
+    }
+  } else {
+    /* select "retlw" instruction */  
+    insn = MEM_USED_MASK | gp_processor_retlw(object->class);
+    for (from = idata->address; from < last; from++) {
+      data = (i_memory_get(idata->data, from) & 0xff);
+      i_memory_put(rom->data, to++, insn | data);
+    }
+  }
+
+}
+
 /* create a program memory section to hold the data */
 
 static void
@@ -445,42 +498,27 @@ _create_rom_section(gp_object_type *object, gp_section_type *section)
 {
   gp_section_type *new = NULL;
   char name[BUFSIZ];
-  int insn;
-  int data;
-  int from;
-  int to;
-  int last;
   
   /* create the new section */
-  strcpy(name, section->name);
-  strcat(name, "_i");
+  strncpy(name, section->name, sizeof(name));
+  strncat(name, "_i", sizeof(name));
   new = gp_coffgen_newsection(name);
-  /* create the ROM contents */
-  to = 0;
-  last = section->address + section->size; 
   if (object->class == PROC_CLASS_PIC16E) {
-    insn = MEM_USED_MASK;
-    for (from = section->address; from < last; from += 2) {
-      data =  (i_memory_get(section->data, from) & 0xff);
-      data |= ((i_memory_get(section->data, from + 1) & 0xff) << 8);
-      i_memory_put(new->data, to++, insn | data);
-    }
     new->size = section->size;
     /* force the section size to be an even number of bytes */ 
     if (section->size & 1) {
       new->size++;
     }
   } else {
-    /* select "retlw" instruction */  
-    insn = MEM_USED_MASK | gp_processor_retlw(object->class);
-    for (from = section->address; from < last; from++) {
-      data = (i_memory_get(section->data, from) & 0xff);
-      i_memory_put(new->data, to++, insn | data);
-    }
     new->size = section->size << 1;
   }
   new->flags = STYP_DATA_ROM;
 
+  /* Copy the data to get the MEM_USED_MASK correct.  It is
+     copied again later to ensure that any patched data is
+     updated in the ROM section */
+  _copy_rom_section(object, section, new);
+
   /* insert the new ROM section after the idata section */
   if (section == object->sections_tail) {
     object->sections_tail = new;
@@ -626,8 +664,8 @@ gp_add_cinit_section(gp_object_type *object, int byte_addr)
     while (section != NULL) {
       if (section->flags & STYP_DATA) {
         /* locate the rom table */
-        strcpy(prog_name, section->name);
-        strcat(prog_name, "_i");
+        strncpy(prog_name, section->name, sizeof(prog_name));
+        strncat(prog_name, "_i", sizeof(prog_name));
         prog_section = gp_coffgen_findsection(object, 
                                               object->sections, 
                                               prog_name);
@@ -949,7 +987,7 @@ gp_cofflink_reloc_assigned(MemBlock *m,
     /* assign the address to this section */
     if (_search_memory(m, byte_addr, section_def->start, section_def->end,
                        size, &current_address, &current_size) == 1) {
-      gp_debug("    sucessful relocation to %#x", current_address);
+      gp_debug("    successful relocation to %#x", current_address);
       if (_has_data(current)) {
         _move_data(current->data, current->address, size, current_address);
       }
@@ -984,7 +1022,9 @@ gp_cofflink_reloc_unassigned(MemBlock *m,
   gp_section_type *current;
   enum section_type type;
   unsigned int size;
+  int first_time;
   int success;
+  int type_avail;
   int i;
   struct symbol *sym;
   struct linker_section *section_def;
@@ -1021,7 +1061,12 @@ gp_cofflink_reloc_unassigned(MemBlock *m,
       gp_debug("  relocating data");
     }
 
+    first_time = 1;
+
+next_pass:
+
     success = 0;
+    type_avail = 0;
     smallest_address = 0;
     smallest_size = 0xffffffff;
 
@@ -1036,7 +1081,7 @@ gp_cofflink_reloc_unassigned(MemBlock *m,
           gp_debug("    size = %#x", current->size);
           gp_debug("    def start = %#x", section_def->start);
           gp_debug("    def end = %#x", section_def->end);
-          
+          type_avail = 1;
           if (_search_memory(m, 
                              byte_addr,
                              section_def->start,
@@ -1056,7 +1101,7 @@ gp_cofflink_reloc_unassigned(MemBlock *m,
 
     /* set the memory used flag for all words in the block */
     if (success == 1) {
-      gp_debug("    sucessful relocation to %#x", smallest_address);
+      gp_debug("    successful relocation to %#x", smallest_address);
       if (_has_data(current)) {
         _move_data(current->data, current->address, size, smallest_address);
       }
@@ -1068,7 +1113,16 @@ gp_cofflink_reloc_unassigned(MemBlock *m,
 
       /* Set the relocated flag */
       current->flags |= STYP_RELOC;
-
+    } else if (gp_relocate_to_shared && (first_time == 1) && (type == databank)) {
+      first_time = 0;
+      type = sharebank;
+      gp_warning("relocation of section \"%s\" failed, relocating to a shared memory location",
+                 current->name);    
+      goto next_pass;
+    } else if (type_avail == 0) {
+      gp_error("linker script has no definition that matches the type of section \"%s\"",
+               current->name);    
+      return;
     } else {
       gp_error("no target memory available for section \"%s\"", current->name);    
       return;
@@ -1144,7 +1198,7 @@ gp_cofflink_fill_pages(gp_object_type *object,
                                  &current_address,
                                  &current_size);
           if (found == 1) {
-            sprintf(fill_name, ".fill_%i", fill_number++);
+            snprintf(fill_name, sizeof(fill_name), ".fill_%i", fill_number++);
             gp_debug("  new section \"%s\" at %#x with size %#x and data %#x", 
                      fill_name,
                      current_address,
@@ -1295,24 +1349,37 @@ check_relative(gp_section_type *section, int org, int argument, int range)
 
 void 
 gp_cofflink_patch_addr(enum proc_class class,
+                       int num_pages,
+                       int num_banks,
                        int bsr_boundary,
                        gp_section_type *section, 
                        gp_symbol_type *symbol,
                        gp_reloc_type *relocation)
 {
   int org;
-  int data;
+  int upper_byte;
+  int current_value;
+  int data = 0;
   int value;
   int offset;
   int write_data = 1;
 
-  /* section address are byte addresses */
-  if (class == PROC_CLASS_PIC16E)
-    org = section->address >> 1;
-  else
-    org = section->address;
-  
-  org += (relocation->address >> 1);
+  if (section->flags & STYP_DATA) {
+    /* It is an initialized data section, so everything is bytes */ 
+    org = section->address + relocation->address;
+    upper_byte = 0;
+  } else {
+    /* section address are byte addresses */
+    if (class == PROC_CLASS_PIC16E) {
+      org = section->address >> 1;
+    } else {
+      org = section->address;
+    }
+    /* the relocation address is always a byte address */
+    org += (relocation->address >> 1);
+    /* the address is odd so put the data in upper byte */
+    upper_byte = relocation->address & 1;
+  }
   
   value = symbol->value + relocation->offset;
 
@@ -1322,8 +1389,8 @@ gp_cofflink_patch_addr(enum proc_class class,
            value);
   
   /* fetch the current contents of the memory */
-  data = i_memory_get(section->data, org);
-  assert(data & MEM_USED_MASK);
+  current_value = i_memory_get(section->data, org);
+  assert(current_value & MEM_USED_MASK);
 
   /* FIXME: Not sure if warnings should be generated for out of range 
             arguments. The linker should make sure values are within
@@ -1334,16 +1401,16 @@ gp_cofflink_patch_addr(enum proc_class class,
     switch(class) {
     case PROC_CLASS_PIC12:
     case PROC_CLASS_SX:
-      data = data | (value & 0xff);
+      data = value & 0xff;
       break;
     case PROC_CLASS_PIC14:
-      data = data | (value & 0x7ff);
+      data = value & 0x7ff;
       break;
     case PROC_CLASS_PIC16:
-      data = data | (value & 0x1fff);
+      data = value & 0x1fff;
       break;
     case PROC_CLASS_PIC16E:
-      data = data | ((value >> 1) & 0xff);
+      data = (value >> 1) & 0xff;
       break;
     default:
       assert(0);
@@ -1353,34 +1420,34 @@ gp_cofflink_patch_addr(enum proc_class class,
     switch(class) {
     case PROC_CLASS_PIC12:
     case PROC_CLASS_SX:
-      data = data | (value & 0x1ff);
+      data = value & 0x1ff;
       break;
     case PROC_CLASS_PIC14:
-      data = data | (value & 0x7ff);
+      data = value & 0x7ff;
       break;
     case PROC_CLASS_PIC16:
-      data = data | (value & 0x1fff);
+      data = value & 0x1fff;
       break;
     case PROC_CLASS_PIC16E:
-      data = data | ((value >> 1) & 0xff);
+      data = (value >> 1) & 0xff;
       break;
     default:
       assert(0);
     }
     break;
   case RELOCT_HIGH:
-    data = data | ((value >> 8) & 0xff);   
+    data = (value >> 8) & 0xff;   
     break; 
   case RELOCT_LOW:
-    data = data | (value & 0xff);   
+    data = value & 0xff;   
     break; 
   case RELOCT_P:
-    data = data | ((value & 0x1f) << 8);   
+    data = (value & 0x1f) << 8;   
     break; 
   case RELOCT_BANKSEL:
     {
       int bank = gp_processor_check_bank(class, value);
-      gp_processor_set_bank(class, bank, section->data, org);
+      gp_processor_set_bank(class, num_banks, bank, section->data, org);
       write_data = 0;
     }
     break;
@@ -1407,14 +1474,14 @@ gp_cofflink_patch_addr(enum proc_class class,
     switch(class) {
     case PROC_CLASS_PIC12:
     case PROC_CLASS_SX:
-      data = data | (value & 0x1f);
+      data = value & 0x1f;
       break;
     case PROC_CLASS_PIC14:
-      data = data | (value & 0x7f);
+      data = value & 0x7f;
       break;
     case PROC_CLASS_PIC16:
     case PROC_CLASS_PIC16E:
-      data = data | (value & 0xff);
+      data = value & 0xff;
       break;
     default:
       assert(0);
@@ -1424,51 +1491,51 @@ gp_cofflink_patch_addr(enum proc_class class,
     switch(class) {
     case PROC_CLASS_PIC12:
     case PROC_CLASS_SX:
-      data = data | (value & 0x1f);
+      data = value & 0x1f;
       break;
     case PROC_CLASS_PIC14:
-      data = data | (value & 0x7f);
+      data = value & 0x7f;
       break;
     default:
       assert(0);
     }
     break; 
   case RELOCT_MOVLR:
-    data = data | ((value << 4) & 0xf0);
+    data = (value << 4) & 0xf0;
     break;
   case RELOCT_MOVLB:
     /* The upper byte of the symbol is used for the BSR.  This is inconsistent
        with the datasheet and the assembler, but is done to maintain
        compatibility with mplink. */
-    data = data | ((value >> 8) & 0xff);
+    data = (value >> 8) & 0xff;
     break;
   case RELOCT_GOTO2:
-    data = data | ((value >> 9) & 0xfff);
+    data = (value >> 9) & 0xfff;
     break;
   case RELOCT_FF1:
-    data = data | (value & 0xfff);
+    data = value & 0xfff;
     break;
   case RELOCT_FF2:
-    data = data | (value & 0xfff);
+    data = value & 0xfff;
     break;
   case RELOCT_LFSR1:
-    data = data | ((value >> 8) & 0xf);
+    data = (value >> 8) & 0xf;
     break;
   case RELOCT_LFSR2:
-    data = data | (value & 0xff);
+    data = value & 0xff;
     break;
   case RELOCT_BRA:
     offset = (value - ((org + 1) << 1)) >> 1;
     check_relative(section, org, offset, 0x7ff);
-    data = data | (offset & 0x7ff);
+    data = offset & 0x7ff;
     break;
   case RELOCT_CONDBRA:
     offset = (value - ((org + 1) << 1)) >> 1;
     check_relative(section, org, offset, 127);
-    data = data | (offset & 0xff);
+    data = offset & 0xff;
     break;
   case RELOCT_UPPER:
-    data = data | ((value >> 16) & 0x3f);   
+    data = (value >> 16) & 0x3f;   
     break; 
   case RELOCT_ACCESS:
     {
@@ -1481,14 +1548,14 @@ gp_cofflink_patch_addr(enum proc_class class,
         a = 1;
       }
 
-      data = data | (a << 8);   
+      data = a << 8;   
     }
     break;
   case RELOCT_PAGESEL_WREG:
   case RELOCT_PAGESEL_BITS:
     {
       int page = gp_processor_check_page(class, value);
-      gp_processor_set_page(class, page, section->data, org);
+      gp_processor_set_page(class, num_pages, page, section->data, org);
       write_data = 0;
     }
     break;
@@ -1510,9 +1577,12 @@ gp_cofflink_patch_addr(enum proc_class class,
     assert(0);
   }
 
+  if (upper_byte)
+    data <<= 8;
+
   /* update memory with the new value */
   if (write_data)
-    i_memory_put(section->data, org, data);
+    i_memory_put(section->data, org, current_value | data);
 
   return;
 }
@@ -1529,11 +1599,16 @@ gp_cofflink_patch(gp_object_type *object,
   gp_symbol_type     *symbol;
   gp_coffsymbol_type *var;
   struct symbol      *sym;
+  int num_pages;
+  int num_banks;
   int bsr_boundary = 0;
   
   if (object->class == PROC_CLASS_PIC16E)  
     bsr_boundary = gp_processor_bsr_boundary(object->processor);
 
+  num_pages = gp_processor_num_pages(object->processor);
+  num_banks = gp_processor_num_banks(object->processor);
+
   gp_debug("patching data with relocated symbols");
 
   while (section != NULL) {
@@ -1555,13 +1630,21 @@ gp_cofflink_patch(gp_object_type *object,
                    symbol->value);
         }
         gp_cofflink_patch_addr(object->class,
-	                       bsr_boundary,
+	                       num_pages,
+                               num_banks,
+                               bsr_boundary,
 			       section,
 			       symbol,
 			       relocation);
 
         relocation = relocation->next;
       }
+
+      /* update the rom with the patched idata sections */
+      if ((section->flags & STYP_DATA) && (section->num_reloc != 0)) {
+        assert(section->next->flags & STYP_DATA_ROM);
+        _copy_rom_section(object, section, section->next);
+      }
     
       /* strip the relocations from the section */
       section->num_reloc = 0;
