@@ -22,6 +22,10 @@ Boston, MA 02111-1307, USA.  */
 #ifndef __GPDIS_H__
 #define __GPDIS_H__
 
-void gp_disassemble(MemBlock *m, int *org, enum proc_class class, char *buffer);
+int gp_disassemble(MemBlock *m,
+                   int org,
+                   enum proc_class class,
+                   char *buffer,
+                   size_t sizeof_buffer);
 
 #endif
