@@ -51,8 +51,12 @@ enum pic_processor {
   pic12ce673,
   pic12ce674,
   pic12cr509a,
+  pic12f508,
+  pic12f509,
   pic12f629,
+  pic12f635,
   pic12f675,
+  pic12f683,
   pic14000,
   pic16c5x,
   pic16cxx,
@@ -148,6 +152,7 @@ enum pic_processor {
   pic16cr72,
   pic16cr83,
   pic16cr84,
+  pic16f505,
   pic16f54,
   pic16f57,
   pic16f627,
@@ -155,6 +160,7 @@ enum pic_processor {
   pic16f628,
   pic16f628a,
   pic16f630,
+  pic16f636,
   pic16f648a,
   pic16f676,
   pic16f684,
@@ -168,6 +174,7 @@ enum pic_processor {
   pic16f767,
   pic16f77,
   pic16f777,
+  pic16f785,
   pic16f818,
   pic16f819,
   pic16f83,
@@ -186,6 +193,10 @@ enum pic_processor {
   pic16f877,
   pic16f877a,
   pic16f88,
+  pic16f913,
+  pic16f914,
+  pic16f916,
+  pic16f917,
   pic16hv540,
   pic17cxx,
   pic17c42,
@@ -213,25 +224,55 @@ enum pic_processor {
   pic18f2220,
   pic18f2320,
   pic18f2331,
+  pic18f2410,
   pic18f242,
+  pic18f2420,
   pic18f2431,
   pic18f2439,
+  pic18f2455,
   pic18f248,
+  pic18f2480,
+  pic18f2510,
+  pic18f2515,
   pic18f252,
+  pic18f2525,
   pic18f2539,
+  pic18f2550,
+  pic18f2520,
   pic18f258,
+  pic18f2580,
+  pic18f2585,
+  pic18f2610,
   pic18f2620,
+  pic18f2680,
+  pic18f2681,
   pic18f4220,
   pic18f4320,
   pic18f4331,
+  pic18f4410,
   pic18f442,
+  pic18f4420,
   pic18f4431,
   pic18f4439,
+  pic18f4455,
   pic18f448,
+  pic18f4480,
+  pic18f4510,
+  pic18f4515,
   pic18f452,
+  pic18f4520,
+  pic18f4525,
   pic18f4539,
+  pic18f4550,
   pic18f458,
+  pic18f4580,
+  pic18f4585,
+  pic18f4610,
   pic18f4620,
+  pic18f4680,
+  pic18f4681,
+  pic18f6410,
+  pic18f6490,
   pic18f6520,
   pic18f6525,
   pic18f6585,
@@ -239,6 +280,8 @@ enum pic_processor {
   pic18f6621,
   pic18f6680,
   pic18f6720,
+  pic18f8410,
+  pic18f8490,
   pic18f8520,
   pic18f8525,
   pic18f8585,
@@ -264,6 +307,8 @@ struct px {
   char *defined_as;
   char *names[MAX_NAMES];
   unsigned long coff_type; 
+  int num_pages;
+  int num_banks;
   char *script;
 };
 
@@ -309,6 +354,8 @@ struct px *gp_find_processor(char *name);
 enum proc_class gp_processor_class(enum pic_processor);
 int gp_processor_bsr_boundary(enum pic_processor processor);
 unsigned long gp_processor_coff_type(enum pic_processor processor);
+int gp_processor_num_pages(enum pic_processor processor);
+int gp_processor_num_banks(enum pic_processor processor);
 enum pic_processor gp_processor_coff_proc(unsigned long coff_type);
 char *gp_processor_name(enum pic_processor processor, unsigned int choice);
 char *gp_processor_coff_name(unsigned long coff_type, unsigned int choice);
@@ -317,10 +364,12 @@ int gp_processor_rom_width(enum proc_class class);
 int gp_processor_check_page(enum proc_class class, int address);
 int gp_processor_check_bank(enum proc_class class, int address);
 int gp_processor_set_page(enum proc_class class, 
+                          int num_pages,
                           int page,
                           MemBlock *m, 
                           int address);
 int gp_processor_set_bank(enum proc_class class, 
+                          int num_banks,
                           int bank,
                           MemBlock *m, 
                           int address);
