@@ -174,21 +174,21 @@ gp_putb32(char *addr, long data)
 }
 
 void
-gp_date_string(char *buffer)
+gp_date_string(char *buffer, size_t sizeof_buffer)
 {
   time_t now;
   struct tm *now_tm;
 
   time(&now);
   now_tm = localtime(&now);
-  sprintf(buffer,
-          "%d-%d-%d  %02d:%02d:%02d",
-          now_tm->tm_mon + 1,
-          now_tm->tm_mday,
-          1900 + now_tm->tm_year,
-          now_tm->tm_hour,
-          now_tm->tm_min,
-          now_tm->tm_sec);
+  snprintf(buffer, sizeof_buffer,
+           "%d-%d-%d  %02d:%02d:%02d",
+           now_tm->tm_mon + 1,
+           now_tm->tm_mday,
+           1900 + now_tm->tm_year,
+           now_tm->tm_hour,
+           now_tm->tm_min,
+           now_tm->tm_sec);
 
   return;
 }
@@ -209,6 +209,22 @@ gp_lower_case(char *name)
   return new;
 }
 
+char *
+gp_upper_case(char *name)
+{
+  char *new;
+  char *ptr;
+
+  ptr = new = strdup(name);
+
+  while (*ptr != '\0') {
+    *ptr = toupper(*ptr);
+    ptr++;
+  }
+
+  return new;
+}
+
 /* linked list functions */
 
 gp_linked_list *
